@@ -5,7 +5,7 @@ use Carp;
 use Scalar::Util qw( refaddr );
 use IO::Select;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 # interal socket information table
 my %SOCK_INFO;
@@ -45,9 +45,10 @@ sub new {
         # load the class
         my $class = 'Net::Proxy::Connector::' . $args->{$conn}{type};
         eval "require $class";
-        croak "Couldn't load $class for '$conn' connector" if $@;
+        croak "Couldn't load $class for '$conn' connector: $@" if $@;
 
         # create and store the Connector object
+        $args->{$conn}{_proxy_} = $self;
         $CONNECTOR{$conn}{ refaddr $self} = $class->new( $args->{$conn} );
         $CONNECTOR{$conn}{ refaddr $self}->set_proxy($self);
     }
@@ -231,8 +232,8 @@ Net::Proxy - Framework for proxying network connections in many ways
     # proxy connections from localhost:6789 to remotehost:9876
     # using standard TCP connections
     my $proxy = Net::Proxy->new(
-        in  => { proto => tcp, port => '6789' },
-        out => { proto => tcp, host => 'remotehost', port => '9876' },
+        in  => { type => tcp, port => '6789' },
+        out => { type => tcp, host => 'remotehost', port => '9876' },
     );
 
     # register the proxy object
@@ -248,8 +249,20 @@ Net::Proxy - Framework for proxying network connections in many ways
 A C<Net::Proxy> object represents a proxy that accepts connections
 and then relays the data transfered between the source and the destination.
 
-The goal of this module is to abstract the different protocols used
-to connect from the proxy to the destination. See L<AVAILABLE CONNECTORS>
+The goal of this module is to abstract the different methods used
+to connect from the proxy to the destination.
+
+A proxy is a program that transfer data across a network boundary           
+between a client and a server. C<Net::Proxy> introduces the concept of         
+"connectors" (implemented as C<Net::Proxy::Connector> subclasses),
+which abstract the server part (connected to the              
+client) and the client part (connected to the server) of the proxy.         
+                                                                            
+This architecture makes it easy to implement specific techniques to
+cross a given network boundary, possibly by using a proxy on one side
+of the network fence, and a reverse-proxy on the other side of the fence.
+
+See L<AVAILABLE CONNECTORS> for details about the existing connectors.
 
 =head1 METHODS
 
@@ -401,6 +414,12 @@ configured host/port.
 This proxy can connect to a TCP server though a web proxy that
 accepts HTTP CONNECT requests.
 
+=head2 dual (C<Net::Proxy::Connector::dual>)
+
+This proxy is a Y-shaped proxy: depending on the client behaviour
+right after the connection is established, it connects it to one
+of two services, handled by two distinct connectors.
+
 =head2 dummy (C<Net::Proxy::Connector::dummy>)
 
 This proxy does nothing. You can use it as a template for writing
@@ -411,6 +430,9 @@ new C<Net::Proxy::Connector> classes.
 This table summarises all the available C<Net::Proxy::Connector>
 classes and the parameters their constructors recognise.
 
+C<N/A> means that the given C<Net::Proxy::Connector> cannot be used
+in that position (either C<in> or C<out>).
+
      Connector  | in parameters   | out parameters
     ------------+-----------------+----------------
      tcp        | host            | host
@@ -423,9 +445,19 @@ classes and the parameters their constructors recognise.
                 |                 | proxy_user
                 |                 | proxy_pass
                 |                 | proxy_agent
+    ------------+-----------------+----------------
+     dual       | host            | N/A
+                | port            |
+                | timeout         |
+                | server_first    |
+                | client_first    |
     ------------+-----------------+----------------
      dummy      | N/A             | N/A
 
+C<Net::Proxy::Connector::dummy> is used as the C<out> parameter for
+a C<Net::Proxy::Connector::dual>, since the later is linked to two
+different connector objects.
+
 =head1 AUTHOR
 
 Philippe 'BooK' Bruhat, C<< <book@cpan.org> >>.
@@ -445,28 +477,14 @@ Here's my own wishlist:
 
 =item *
 
-port C<connect-tunnel> to use C<Net::Proxy>.
-
-This requires writing C<Net::Proxy::Connector::connect>.
-
-=item *
-
-port C<sslh> (unreleased reverse proxy that can listen on a port and
-proxy to a SSH server or a HTTPS server depending on the client) to
-use C<Net::Proxy>.
-
-This requires writing C<Net::Proxy::Connector::dual>.
-
-=item *
-
-write a script fully compatible with GNU httptunnel
-(L<http://www.nocrew.org/software/httptunnel.html>.
+Write a script fully compatible with GNU httptunnel
+(L<http://www.nocrew.org/software/httptunnel.html>).
 
 This requires writing C<Net::Proxy::Connector::httptunnel>.
 
 =item *
 
-enhance the httptunnel protocol to support multiple connections
+Enhance the httptunnel protocol to support multiple connections.
 
 This requires writing C<Net::Proxy::Connector::httptunnel2>
 (or whatever I may call it then).
@@ -478,9 +496,31 @@ C<Net::Proxy::Connector::FEP>. This RFC was published on April 1, 2001.
 
 =item *
 
+Implement DNS tunnel connectors (see
+L<http://savannah.nongnu.org/projects/nstx/>,
+OzymanDNS, L<http://www.doxpara.com/slides/BH_EU_05-Kaminsky.pdf>.
+L<http://thomer.com/howtos/nstx.html>).
+
+=item *
+
+Implement ICMP tunnel connectors (see
+L<http://www.linuxexposed.com/Articles/Hacking/Case-of-a-wireless-hack.html>,
+L<http://sourceforge.net/projects/itun>,
+L<http://www.cs.uit.no/~daniels/PingTunnel/>,
+L<http://thomer.com/icmptx/>).
+
+Since this does not imply TCP connections, it's for a distant future.
+
+=item *
+
 Add support for filters, so that the data can be transformed on the fly
 (could be useful to deceive intrusion detection systems, for example).
 
+=item *
+
+Look for inspiration in the I<Firewall-Piercing HOWTO>, 
+at L<http://fare.tunes.org/files/fwprc/>.
+
 =back
 
 =head1 COPYRIGHT
