@@ -14,6 +14,7 @@ sub new {
     my ( $class, $args ) = @_;
     my $self = bless $args ? {%$args} : {}, $class;
     $self->init() if $self->can('init');
+    delete $self->{_proxy_}; # this link back is now unnecessary
     return $self;
 }
 
@@ -51,22 +52,31 @@ sub new_connection_on {
     Net::Proxy->watch_sockets($sock);
 
     # connect to the destination
-    my $proxy = $self->get_proxy();
-    my $out   = $proxy->out_connector();
-    my $peer  = eval { $out->connect(); };
-    if ($@) { # connect() dies if the connection fails
+    my $out = $self->get_proxy()->out_connector();
+    $out->_out_connect_from($sock);
+
+    # update the stats
+    $self->get_proxy()->stat_inc_opened();
+    return;
+}
+
+sub _out_connect_from {
+    my ( $self, $sock ) = @_;
+
+    my $peer = eval { $self->connect(); };
+    if ($@) {    # connect() dies if the connection fails
         $@ =~ s/ at .*?\z//s;
-        carp "connect() failed with error '$@'";
-        Net::Proxy->close_sockets( $sock );
+        warn "connect() failed with error '$@'\n";
+        Net::Proxy->close_sockets($sock);
         return;
     }
     if ($peer) {    # $peer is undef for Net::Proxy::Connector::dummy
         Net::Proxy->watch_sockets($peer);
-        Net::Proxy->set_connector( $peer, $out );
+        Net::Proxy->set_connector( $peer, $self );
         Net::Proxy->set_peer( $peer, $sock );
         Net::Proxy->set_peer( $sock, $peer );
     }
-    $proxy->stat_inc_opened();
+
     return;
 }
 
@@ -81,7 +91,7 @@ sub raw_read_from {
 
     # check for errors
     if ( not defined $read ) {
-        carp sprintf( "Read undef from %s:%s (Error %d: %s)\n",
+        warn sprintf( "Read undef from %s:%s (Error %d: %s)\n",
             $sock->sockhost(), $sock->sockport(), $!, "$!" );
         $close = 1;
     }
@@ -101,7 +111,7 @@ sub raw_write_to {
     my ($self, $sock, $data) = @_;
     my $written = $sock->syswrite( $data );
     if( ! defined $written ) {
-        carp sprintf("Read undef from %s:%s (Error %d: %s)\n",
+        warn sprintf("Read undef from %s:%s (Error %d: %s)\n",
                      $sock->sockhost(), $sock->sockport(), $!, "$!");
     }
     return;
@@ -112,7 +122,7 @@ sub raw_listen {
     my $self = shift;
     my $sock = IO::Socket::INET->new(
         Listen    => 1,
-        LocalAddr => $self->{host},
+        LocalAddr => $self->{host} || 'localhost',
         LocalPort => $self->{port},
         Proto     => 'tcp',
         ReuseAddr => 1,
